$!
$! LCKQUE.COM
$!
$! Summarize lock queues in the cluster
$!							K.Parris 01/2000
$!	Before running this to gather lock-queue data, you must first run it
$!	with a P1 parameter of "BUILD" to build an associated data-gathering
$!	program called LCKQUE.MAR (the source for which is contained in
$!	this procedure).  You'll need to build (compile or assemble & link) once
$!	for each of the different CPU architectures and VMS versions in the
$!	cluster (because it looks at system data structures).  The Macro
$!	code included works on both VAX and Alpha architectures.
$!
$!	To gather lock-queue data, run this procedure without any parameters.
$!	It generates a listing of lock queues, which it also places into a
$!	a file with a name of the format LCKQUE_<date/time>.RPT.
$!
$!	The procedure requires privileges OPER (for SYSMAN) and CMKRNL and
$!	CMEXEC for the LCKQUE.MAR program.
$!
$!	To minimize risk and intrusion, the LCKQUE program does not raise
$!	IPL and does not acquire spinlocks as it traverses data structures
$!	and gathers lock-queue data.  Although it uses only Executive mode
$!	(Kernel mode on a few VMS versions starting at 7.2 where Exec-mode
$!	access to Lock Manager data structures was temporarily restricted)
$!	an inner-mode handler is present to catch any possible ACCVIOs or
$!	such and prevent a system crash in the kernel-mode case (an Exec-mode
$!	bugcheck resulting in process termination is the worst-case scenario
$!	possible with Exec-mode access (unless the SYSGEN parameters
$!	BUGCHECKFATAL or SYSTEM_CHECK are set to 1, of course -- but we check
$!	for that possibility and don't run the program in that case).
$!
$!	Since we are traversing data structures without raising IPL or
$!	taking out spinlocks (so as to avoid contributing to MP_Synch
$!	time), occasinally the LCKQUE program will go off into the weeds
$!	or get into an infinite loop because the data structures changed
$!	underneath it.  Such cases are caught by a SYSMAN TIMEOUT or the exit
$!	handler catching the ACCVIO and ending the program, and there is code
$!	present here to detect the incomplete data report in that case and
$!	warn the user if that has occurred.
$!
$!	By default, this procedure filters out commonly-encountered "deadman"
$!	locks (those which always have lock queues as a matter of course).
$!	You can change that behavior and report on all lock queues if you wish,
$!	by specifying "NOFILTER" as the P1 parameter.
$!
$!		                                       Version 1.2, April 2003
$! Modification history:
$!
$!	V1.2	Incorporated LCKQUE.MAR, BUILD_LCKQUE.COM,	April 2003
$!		and LCKQUE_RUN.COM into this one procedure.
$!	V1.1	Improved filtering with sub-resource names	March 2003
$!	V1.0	Version as of V7.3-1				September 2002
$!	V0.1	Initial version					January 2000
$!
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$!                       or keith.parris@hp.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$!
$	proc = f$element(0,";",f$environment("procedure"))
$	proc_dev = f$parse(proc,,,"DEVICE")
$	proc_dir = proc_dev + f$parse(proc,,,"DIRECTORY")
$!
$! Filtering of common deadman locks (at least the ones I've come across) is
$! enabled by default.  This can be controlled by a P1 parameter value of
$! "FILTER" or "NOFILTER".
$	filtering = "TRUE"
$	if p1 .eqs. "FILTER" .or. p1 .eqs. "NOFILTER"
$	then
$		if p1 .eqs. "FILTER" then filtering = "TRUE"
$		if p1 .eqs. "NOFILTER" then filtering = "FALSE"
$		p1 = ""
$	endif
$!!! filtering = "FALSE"	!!!for testing
$!
$	if p1 .nes. ""
$	then
$		if p1 .nes. "BUILD" .and. p1 .nes. "LCKQUE_RUN"
$		then
$			write sys$output "Error in P1 parameter: ",p1
$			exit
$		endif
$		if p1 .nes. "" then goto 'p1'
$	endif
$! Check for the privileges we'll need to drive this process -- most
$! importantly, OPER for SYSMAN here, and CMKRNL to use SDA on each node
$! to gather data
$!
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs  = "," + f$getjpi("","CURPRIV")  + ","
$! Check for OPER privilege and turn it on if need be
$	OPER_TURNED_ON = "F"	!Assume process already had OPER privilege turned on
$	if f$locate(",OPER,",cur_privs) .eq. f$length(cur_privs)
$	then	!OPER privilege is not presently turned on
$		if f$locate(",OPER,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(OPER)
$			OPER_TURNED_ON = "T"
$		else
$			write sys$output -
		    "This procedure requires OPER privilege to run SYSMAN."
$			exit
$		endif
$	endif
$! Check for CMKRNL privilege rights
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMKRNL is authorized
$		else
$			write sys$output -
	"This procedure requires CMKRNL privilege to run the LCKQUE program."
$			goto cleanup
$		endif
$	endif
$! Check for CMEXEC privilege rights
$	if f$locate(",CMEXEC,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMEXEC privilege is not presently turned on
$		if f$locate(",CMEXEC,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMEXEC is authorized
$		else
$			write sys$output -
	"This procedure requires CMEXEC privilege to run the LCKQUE program."
$			goto cleanup
$		endif
$	endif
$!
$! Make sure they've built the program first
$!
$	if (f$search("''proc_dir'LCKQUE.EXE_*_*")  .eqs. "")
$	then
$		write sys$output -
"The LCKQUE program needs to be built.  Execute $ @LCKQUE BUILD"
$		write sys$output -
"once on each combination of architecture and version of VMS in the cluster."
$		write sys$output -
"This cluster contains the following architecture(s) and version(s):"
$		mcr sysman set environment/cluster
do write sys$output "OpenVMS ",f$getsyi("ARCH_NAME")," ",f$getsyi("VERSION")
exit
$		goto cleanup
$	endif
$!
$! Gather lock-queue data
$!
$	if_debug = "!"	!Set to "!" to disable debugging code, "" to enable
$!!!	if_debug = ""	!Set to "!" to disable debugging code, "" to enable
$!
$! Check for the privileges we'll need to drive this process -- most
$! importantly, OPER for SYSMAN here, and CMKRNL to use SDA on each node
$! to gather data
$!
$	auth_privs = "," + f$getjpi("","AUTHPRIV") + ","
$	cur_privs  = "," + f$getjpi("","CURPRIV")  + ","
$! Check for OPER privilege and turn it on if need be
$	OPER_TURNED_ON = "F"	!Assume process already had OPER privilege turned on
$	if f$locate(",OPER,",cur_privs) .eq. f$length(cur_privs)
$	then	!OPER privilege is not presently turned on
$		if f$locate(",OPER,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then
$			set process/privilege=(OPER)
$			OPER_TURNED_ON = "T"
$		else
$			write sys$output -
		    "This procedure requires OPER privilege to run SYSMAN."
$			exit
$		endif
$	endif
$! Check for CMKRNL privilege rights
$	if f$locate(",CMKRNL,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMKRNL privilege is not presently turned on
$		if f$locate(",CMKRNL,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMKRNL is authorized
$		else
$			write sys$output -
	"This procedure requires CMKRNL privilege to run the LCKQUE program."
$			goto cleanup
$		endif
$	endif
$! Check for CMEXEC privilege rights
$	if f$locate(",CMEXEC,",cur_privs) .eq. f$length(cur_privs)
$	then	!CMEXEC privilege is not presently turned on
$		if f$locate(",CMEXEC,",auth_privs) .ne. f$length(auth_privs) .or. -
		   f$locate(",SETPRV,",auth_privs) .ne. f$length(auth_privs)
$		then	!CMEXEC is authorized
$		else
$			write sys$output -
	"This procedure requires CMEXEC privilege to run the LCKQUE program."
$			goto cleanup
$		endif
$	endif
$!
$	pid = f$getjpi("","PID")
$	temp_file := LCKQUE_'pid'_temp.temp_file
$!!!'if_debug' temp_file := LCKQUE_temp.temp_file	!!! for testing
$	delete := delete
$	date_time = f$time()	!Grab a timestamp
$	date = f$cvtime(date_time,"COMPARISON","DATE") - "-" - "-"
$	time = f$cvtime(date_time,"COMPARISON","TIME")
$	hhmm = f$element(0,":",time) + f$element(1,":",time)	!HoursMinutes
$	output_file := LCKQUE_'date'_'hhmm'.RPT
$	output_dir = proc_dir	!!!:= DISK$APP3:[KPARRIS.LCKACT]
$!!!	show symbol date_time	!Time-stamp the output
$!!!	show symbol output_file	!Show output file specification
$	spaces = -	!64 blank spaces
    "                                                                "
$!   1234567890123456789012345678901234567890123456789012345678901234
$!            1         2         3         4         5         6
$	spaces = spaces + spaces	!128 spaces
$	spaces = spaces + spaces	!256 spaces
$!!!'if_debug' goto temp_start	!!!@@@!!!
$! Gather lock queue lengths from all nodes in the cluster
$	close/nolog com
$	open/write com 'temp_file'_com
$	write com "$ SET NOON"
$	write com "$ MCR SYSMAN SET ENVIRONMENT/CLUSTER"
$! Since we aren't raising IPL or taking out spinlocks (so as not to conribute
$! to MPSYNCH time), we'll occasionally have data structures moved out from
$! underneath us and end up off in the weeds, occasionally with old data
$! structures that point in circles, resulting in an 'infinite' loop.  Having a
$! timeout prevents a loop from actually being infinite.  Since the LCKQUE
$! program normally prints the message "LCKQUE done." when it finishes
$! normally, we can tell if data-collection was successful or not, and let
$! the user know if there was a problem.
$!!!	write com "SET TIMEOUT 0:0:3"	!Give up after 3 seconds
$	write com "SET TIMEOUT 0:0:10"	!Give up after 10 seconds
$	write com "SET PROFILE/PRIV=(CMKRNL,CMEXEC)"
$	write com "DO @",proc," LCKQUE_RUN"
$	write com "EXIT"
$	write com "$ EXIT 1"
$	close com
$!!! show time	!Time-stamp the output
$	write sys$output "Gathering lock queues from all cluster nodes..."
$ 'if_debug' type/page 'temp_file'_com
$	@'temp_file'_com/out='temp_file'_lis
$	delete 'temp_file'_com;*
$ 'if_debug' type/page 'temp_file'_lis
$ temp_start:	!!! for testing
$!!!	write sys$output ""
$!!! show time	!Time-stamp the output again
$	write sys$output "Post-processing data..."
$!
$! Output from LCKQUE.EXE looks like this:
$!    Lock queue record:
$!         nnn nnnnnnnnnn nnnnnnnnnn nnn xxx...<hex for exactly 32 bytes>...xxx
$!          |      |          |       |       |
$!Fields: Depth ConvQueue WaitQueue RsnLen  RsnX
$! where ConvQueue is the number of lock blocks hanging off the conversion
$! queue waiting for a conversion request to be granted, and WaitQueue is the
$! number of new lock request lock blocks hanging off the wait queue.
$! If the resource is not a root resource, one or more parent resource name
$! records immediately precedes the resource name with queue lengths:
$!    Parent resource name record:
$!         nnn (Parent resource:)    nnn xxx...<hex for exactly 32 bytes>...xxx
$!          |      |          |       |       |
$!Fields: Depth ConvQueue WaitQueue RsnLen  RsnX
$! One parent resource record is output for each level above the resource on
$! which we see queues (to fully specify the resource hierarchy).  The root
$! resource name is output first.
$!
$! (Note that since the LCKQUE program dumps 32 bytes in the resource name field
$! regardless of how many are significant, this implies some of the trailing
$! bytes (positioned at the left-hand side of the RsnX variable) are
$! unpredictable.  The RsnLen field indicates how may bytes are valid.  We
$! create a symbol Rsn_Hex with only the valid hexadecimal bytes.)
$!
$	close/nolog lis
$	open/read lis 'temp_file'_lis
$	close/nolog ls2
$	open/write ls2 'output_file'
$	write ls2 "Time: ",f$extract(0,2,hhmm),":",f$extract(2,2,hhmm)
$	node_completed = "TRUE"	!Avoid false alarm the first nodename we come across
$ loop_lis:
$	read/end=eof_lis lis rec
$	if f$extract(0,44,rec) .eqs. -
		"%SYSMAN-I-OUTPUT, command execution on node "
$	then
$		! See if LCKQUE on the last node completed successfully
$		if .not. node_completed
$		then
$			write ls2 ""
$			write ls2 " ***"
$			write ls2 " *** Node ",node," data collection failed."
$			write ls2 " ***"
$		endif
$		node = f$extract(44,6,rec)
$		node_heading = "FALSE"	!Haven't yet output nodename heading
$		node_completed = "FALSE"
$		goto loop_lis
$	endif
$	if rec .eqs. "LCKQUE done."
$	then	! This indicates LCKQUE.EXE ran to successful completion
$		node_completed = "TRUE"
$		goto loop_lis
$	endif
$	if f$length(rec) .lt. 25 then goto loop_lis	!Not a data record -->
$! Check Depth field to weed out SYSMAN messages from log  (it's a 3-digit #)
$	if f$locate(f$extract(2,1,rec),"0123456789") .eq. 10 then -
		goto loop_lis	!Not a data record -->
$!
$! Here we have a resource-name record.
$	depth = 0	!Expected depth of resource here
$!
$! Read all the records for this tree of resources and all its sub-resources,
$! and remember the resource names at each levl.  That way, we'll have all the
$! information we need to do filtering of resource names across multiple levels.
$!
$! Parent resource records look like this:
$!         nnn (Parent resource:)    nnn xxx...<hex for exactly 32 bytes>...xxx
$!          |      |          |       |       |
$!Fields: Depth ConvQueue WaitQueue RsnLen  RsnX
$! instead of:
$!         nnn nnnnnnnnnn nnnnnnnnnn nnn xxx...<hex for exactly 32 bytes>...xxx
$!          |      |          |       |       |
$!Fields: Depth ConvQueue WaitQueue RsnLen  RsnX
$! so the ConvQueue variable will contain "(Parent" and the WaitQueue variable
$! will contain "resource:)" if we aren't at a leaf resource (where the lock
$! queue(s) were detected).
$!
$!
$! We come back to loop here for each resource name of a parent lock, on our
$! way down a sub-resource chain to the resource on which a lock queue was found
$!
$ subres_loop:	!Following a chain of sub-resources
$!!!'if_debug' show symbol rec	!!! for testing
$	crec = f$edit(rec,"COMPRESS,TRIM")
$	fld_depth = f$element(0," ",crec)	!Depth reported by program
$ if fld_depth .ne. depth
$ then
$  write sys$output -	!Mismatch   (should never happen in practice)
"*** Depth field: ",fld_depth,", Actual depth: ",depth,". Field overridden. ***"
$ endif
$	ConvQueue = f$element(1," ",crec)
$	WaitQueue = f$element(2," ",crec)
$	RsnLen = f$integer(f$element(3," ",crec)) !Actual resource name length (bytes)
$	RsnX = f$element(4," ",crec)	!Resource name (fixed 32 bytes)
$	hxrlen = RsnLen*2	!Length of resource name in hex-digit form
$	Rsn_Hex = f$extract(64-hxrlen,hxrlen,RsnX)	!Trim leading junk off
$	RsnLen_'depth' = RsnLen		!Remember parent resource names by level
$	Rsn_Hex_'depth' = Rsn_Hex
$!
$! Convert the resource name to its ASCII equivalent, replacing non-printable
$! characters with "." characters.  Also save hexadecimal values for each byte.
$	Rsn_ASCII = ""
$	i = 1
$ ascii_loop:
$	HexByte = f$extract(hxrlen-(2*i),2,Rsn_Hex)
$	HexByte_'i' = HexByte	!An 'array' of sorts for the hex byte values
$	HexByte_'i'_'depth' = HexByte	!Remember this for each depth level
$	byte = %x'HexByte'
$!!not used!!	byte_'i' = byte		!An 'array' of sorts for the byte values
$!!not used!!	byte_'i'_'depth' = byte	!Remember this for each depth level
$	cbyte = "."
$	if byte .ge. 32 .and. byte .lt. 127 then cbyte[0,7] = byte  !Printable
$	Rsn_ASCII = Rsn_ASCII + cbyte
$	i = i + 1
$	if i .le. RsnLen then goto ascii_loop
$!
$	Rsn_ASCII_'depth' = Rsn_ASCII		!Remember parent resource names by level
$!
$	if ConvQueue .eqs. "(Parent" !.and. WaitQueue .eqs. "Resource:)"
$	then	!Parent lock
$		read/end=eof_lis lis rec	!Read the next sub-resource record
$		depth = depth + 1	!We're diving down another level deep
$		goto subres_loop	!Parse the next level of resource name
$	endif
$!
$! Here, we have all the information about the lock on which queuing was
$! detected. 
$!
$!
$! Filter out dead-man locks that are always waiting and are thus of no interest
$!
$ if .not. filtering then goto past_filter  !!!Uncomment to see every lock queue, even standing ones
$!
$! In pattern-matching on resource names, we have these symbols available:
$!   RsnLen_'depth'	!Resource name length in bytes
$!   Rsn_ASCII_'depth'	!Resource name, printable ASCII text (unprintable
$!			 characters replaced by periods (.)
$!   Rsn_HEX_'depth'	!Resource name, hexadecimal format, least-significant
$!			 bytes at the right-hand end, like $DUMP format displays
$!   HexByte_'i'_'depth'!Hexadecimal bytes, indexed [1..RsnLen] by _'i'_
$! Here 'depth' is zero for a root resource, and increments by 1 for each
$! sub-resource level.
$!
$! Look for root-level deadman locks, and skip them.
$	if depth .eq. 0
$	then	!Have a root lock here
$! VIOC/XFC
$	    if f$extract(0,5,Rsn_ASCII_0) .eqs. "VCC$v" then goto loop_lis
$! 'CACHE$cm' [1:8], length 24, wq 0, depth 0
$!'CACHE$cmSYS2        ....' 000000F92020202020202020325359536D63244548434143
$! Convert queue: 5
$	    if f$extract(0,8,Rsn_ASCII_0) .eqs. "CACHE$cm" .and. -
		RsnLen_0 .eq. 24 .and. WaitQueue .eq. 0 then goto loop_lis
$! Host-Based Volume Shadowing volume watcher lock
$! '$DSA' [1:4], '_$WATCHR' [8:16], length 16, cq 0, depth 0
$!'$DSA0204_$WATCHR' 524843544157245F3430323041534424
$! Wait queue: 6
$!
$!'$DSA0100_$COPIER' 524549504F43245F3030313041534424
$! Wait queue: 2
$	    if f$extract(0,4,Rsn_ASCII_0) .eqs. "$DSA" .and. -
        	(f$extract(8,8,Rsn_ASCII_0) .eqs. "_$WATCHR" .or. -
        	 f$extract(8,8,Rsn_ASCII_0) .eqs. "_$COPIER") .and. -
		RsnLen_0 .eq. 16 .and. ConvQueue .eq. 0 then goto loop_lis
$! IOGEN
$!'IOGEN$_400' 3030345F244E45474F49
$! Wait queue: 1
$!
$!'IOGEN$_1' 315F244E45474F49
$! Wait queue: 17
$	    if f$extract(0,7,Rsn_ASCII_0) .eqs. "IOGEN$_" .and. -
		RsnLen_0 .gt. 7 .and. ConvQueue .eq. 0 then goto loop_lis
$! Audit server
$!'AUDRSV$J' [1:8], length 30, wq 0
$!'AUDRSV$J.......NEWAUDIT    ...' 00000020202020544944554157454E0200000004001C4A`
$! Convert queue: 6
$	    if f$extract(0,8,Rsn_ASCII_0) .eqs. "AUDRSV$J" .and. -
		RsnLen_0 .eq. 30 then goto loop_lis
$!
$	    if f$extract(0,7,Rsn_ASCII_0) .eqs. "PSCDNS_" .and. -
		RsnLen_0 .ge. 8 then goto loop_lis
$!
$! 'UCX$INETACP_' in [1:12], length 16, cq 0, depth 0
$! 'UCX$INETACP_....' 02E6B4C65F50434154454E4924584355
$!  Convert queue: 0, Wait queue: 1
$	    if f$extract(0,12,Rsn_ASCII_0) .eqs. "UCX$INETACP_" .and. -
		RsnLen_0 .eq. 16 .and. -
		ConvQueue .eq. 0 then goto loop_lis
$!
$! Hierarchical Storage Manager
$!'HSM$SHELF_SERVER' 5245565245535F464C454853244D5348
$! Convert queue: 1, Wait queue: 3
$	    if Rsn_ASCII_0 .eqs. "HSM$SHELF_SERVER" then goto loop_lis
$!
$! DECscheduler (?)
$	    if f$extract(0,7,Rsn_ASCII_0) .eqs. "NSCHED_"
$	    then
$!'NSCHED_MASTER' 52455453414D5F44454843534E
$! Wait queue: 5
$		if Rsn_ASCII_0 .eqs. "NSCHED_MASTER" .and. ConvQueue .eq. 0 then -
			goto loop_lis
$! 'NSCHED__' in [1:8], length varies depending on nodename, cq 0, wq 1, depth 0
$! 'NSCHED__BBC200' 3030324342425F5F44454843534E
$!  Convert queue: 0, Wait queue: 1
$		if f$extract(0,8,Rsn_ASCII_0) .eqs. "NSCHED__" .and. -
			RsnLen_0 .gt. 8 .and. -
			ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$! 'NSCHED_541092781' 3138373239303134355F44454843534E
$! Wait queue: 1
$		if RsnLen_0 .eq. 16 .and. -
			ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$	    endif
$!
$! Pathworks (aka Advanced Server)
$!'PWRK$DMN_UAS' 5341555F4E4D44244B525750
$! Wait queue: 1
$	    if Rsn_ASCII_0 .eqs. "PWRK$DMN_UAS" .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$!'PWRK$LICENSE_SERVER' 5245565245535F45534E4543494C244B525750
$! Wait queue: 1
$	    if Rsn_ASCII_0 .eqs. "PWRK$LICENSE_SERVER" .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$!'PWRK$node_departure_1' 315F6572757472617065645F65646F6E244B525750
$! Wait queue: 1
$	    if f$extract(0,20,Rsn_ASCII_0) .eqs. "PWRK$node_departure_" .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$!'PWRK$LMSRV_ALIVE_2BE0046A......' 00000000000041363430304542325F4556494C415F565`
$! Convert queue: 1, Wait queue: 1
$	    if f$extract(0,17,Rsn_ASCII_0) .eqs. "PWRK$LMSRV_ALIVE_" .and. -
		RsnLen_0 .eq. 31 then goto loop_lis
$!'PWRK$BROWSER' 524553574F5242244B525750
$! Wait queue: 1
$	    if Rsn_ASCII_0 .eqs. "PWRK$BROWSER" .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$!'PWRK$PRINTSYNC.................' 0000000000000000000000000000000000434E5953544`
$! Wait queue: 1
$	    if f$extract(0,14,Rsn_ASCII_0) .eqs. "PWRK$PRINTSYNC" .and. -
		RsnLen_0 .eq. 31 then goto loop_lis
$! 'EPC$BE_' in [1:7], length 15, conv que 0, wait que 1, depth 0
$! 'EPC$BE_202009AE' 45413930303230325F454224435045
$!  Convert queue: 0, Wait queue: 1
$	    if f$extract(0,7,Rsn_ASCII_0) .eqs. "EPC$BE_" .and. -
		RsnLen_0 .eq. 15 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$! DECwindows
$!'DECW$CLIENT_21E00446_00EE_0' 305F454530305F36343430304531325F544E45494C4324574`
$! Wait queue: 1
$! or
$!'DECW$SERVER_21E00446_00EE_0' 305F454530305F36343430304531325F52455652455324574`
$! Wait queue: 1
$	    if f$extract(0,5,Rsn_ASCII_0) .eqs. "DECW$" .and. -
		(f$extract(5,7,Rsn_ASCII_0) .eqs. "SERVER_" .or. -
		 f$extract(5,7,Rsn_ASCII_0) .eqs. "CLIENT_") .and. -
		f$extract(20,1,Rsn_ASCII_0) .eqs. "_" .and. -
		f$extract(25,1,Rsn_ASCII_0) .eqs. "_" .and. -
		RsnLen_0 .eq. 27 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$! Rdb:
$! 000000DD hex[1;4], 'T' in [5] position, length 16, cq 0, wq 1, depth 0
$! '....T...:.  ....' 000000012020083A00000054000000DD
$!  Convert queue: 0, Wait queue: 1
$	    if f$extract((2*RsnLen_0)-8,8,Rsn_Hex_0) .eqs. "000000DD" .and. -
		f$extract(4,1,Rsn_ASCII_0) .eqs. "T" .and. -
		RsnLen_0 .eq. 16 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$! Apache web server (Compaq Secure Web Server, or CSWS)
$!'APACHE$PARENT_24E004C7' 37433430304534325F544E4552415024454843415041
$!'' 37433430304534325F544E4552415024454843415041
$! Wait queue: 7
$	    if f$extract(0,14,Rsn_ASCII_0) .eqs. "APACHE$PARENT_" .and. -
		RsnLen_0 .eq. 22 .and. -
		ConvQueue .eq. 0 then goto loop_lis
$!
$! 
$! 'ndm$$swX' in [1:8), length 31, conv que 0, wait que 1, depth 0
$! 'ndm$$swX^P.......  ......BBC200' 303032434242D71EABD0A99D202009D50000000006505E58777324246D646E
$!  Convert queue: 0, Wait queue: 1
$	    if f$extract(0,8,Rsn_ASCII_0) .eqs. "ndm$$swX" .and. -
		RsnLen_0 .eq. 31 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$! 
$! 'ndm$$tcp_tlxi_' in [1:14], length 25, depth 0
$! 'ndm$$tcp_tlxi_NDM202009D6' 36443930303230324D444E5F69786C745F70637424246D646E
$!  Convert queue: 0, Wait queue: 1
$	    if f$extract(0,14,Rsn_ASCII_0) .eqs. "ndm$$tcp_tlxi_" .and. -
		RsnLen_0 .eq. 25 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .eq. 1 then goto loop_lis
$!
$	endif	!End of Depth 0
$!
$! Handle multi-level deadman locks
$!
$	if depth .eq. 1
$	then
$! Queue manager deadman lock
$! 'QMAN$MSR_' [1:9], length 30, cq 0, depth 1
$!  'QMAN$JBC_ALIVE_' [1:15], length 17
$!'QMAN$MSR_DSA60..........N.....' 00000001004E0000000000000000000030364153445F52`
$! 'QMAN$JBC_ALIVE_01' 31305F4556494C415F43424A244E414D51
$!  Wait queue: 1
	    if f$extract(0,9,Rsn_ASCII_0) .eqs. "QMAN$MSR_" .and. -
			RsnLen_0 .eq. 30 .and. ConvQueue .eq. 0 .and. -
		f$extract(0,15,Rsn_ASCII_1) .eqs. "QMAN$JBC_ALIVE_" .and. -
			RsnLen_1 .eq. 17 then goto loop_lis
$! DECnet
$! 'CLU$NETACP_' [1:11], cq 0
$!  'IDL_' [1:4]
$!'CLU$NETACP_29067' 37363039325F50434154454E24554C43
$! 'IDL_3' 335F4C4449
$!  Wait queue: 4
$	    if f$extract(0,11,Rsn_ASCII_0) .eqs. "CLU$NETACP_" .and. -
		 f$extract(0,4,Rsn_ASCII_1) .eqs. "IDL_" .and. -
		ConvQueue .eq. 0 then goto loop_lis
$! Rdb:
$! 000000DD hex[1:4], length 28, 'D' in [5] position
$!  'I' in [1] position, hex 000000 in [2:4], length 6, cq 0, depth 1
$! '....D....B05_DB03DBXD.......' 00000002011D0044584244333042445F3530420200000044000000DD
$!  'I.....' 000900000049
$!   Convert queue: 0, Wait queue: 82
$	    if f$extract((2*RsnLen_0)-8,8,Rsn_Hex_0) .eqs. "000000DD" .and. -
		f$extract(4,1,Rsn_ASCII_0) .eqs. "D" .and. -
			RsnLen_0 .eq. 28 .and. -
		f$extract(0,1,Rsn_ASCII_1) .eqs. "I" .and. -
			RsnLen_1 .eq. 6 .and. -
		f$extract((2*RsnLen_1)-8,6,Rsn_Hex_1) .eqs. "000000" .and. -
		ConvQueue .eq. 0 then goto loop_lis
$! 
$! 'JCCLML' (length 6)
$!  'SUBRDB' (database name, and thus length, varies), cq 0, wq 1-2, depth 1
$! 'JCCLML' 4C4D4C43434A
$!  'SUBRDB09' 3930424452425553
$!   Convert queue: 0, Wait queue: 1
$	    if Rsn_ASCII_0 .eqs. "JCCLML" .and. -
-!2nd resource name is Database name and may vary...
-!		f$extract(0,6,Rsn_ASCII_1) .eqs. "SUBRDB" .and. -
-!			RsnLen_1 .eq. 8 .and. -
		ConvQueue .eq. 0 .and. WaitQueue .le. 2 then goto loop_lis
$!
$	endif	!End of Depth 1
$!
$	if depth .eq. 2
$	then
$! 
$! 'NET$Alias_' in [1:10], length 16
$!  'ALIAS$Member_' in [1:13], length 17
$!   'ALIAS$Departure', (length 15), cq 0, depth 2
$! 'NET$Alias_......' 04A9000400AA5F7361696C412454454E
$!  'ALIAS$Member_....' 0001000A5F7265626D654D245341494C41
$!   'ALIAS$Departure' 657275747261706544245341494C41
$!    Convert queue: 0, Wait queue: 9
$	    if f$extract(0,10,Rsn_ASCII_0) .eqs. "NET$Alias_" .and. -
			RsnLen_0 .eq. 16 .and. -
		f$extract(0,13,Rsn_ASCII_1) .eqs. "ALIAS$Member_" .and. -
			RsnLen_1 .eq. 17 .and. -
		Rsn_ASCII_2 .eqs. "ALIAS$Departure" .and. -
		ConvQueue .eq. 0 then goto loop_lis
$! 
$! 'JCCLML - ' in [1:9], length 15
$!  'JCC$' in [1:4], '_T' in [11:12], length 12
$!   'JCC$Deadman' (length 11), cq 0, depth 2
$! 'JCCLML - BBC301' 313033434242202D204C4D4C43434A
$!  'JCC$SUBR01_T' 545F3130524255532443434A
$!   'JCC$Deadman' 6E616D646165442443434A
$!    Convert queue: 0, Wait queue: 1
$!'JCCLML - BBC305' 353033434242202D204C4D4C43434A
$! 'JCC$SUBR05_T' 545F3530524255532443434A
$!  'JCC$Deadman' 6E616D646165442443434A
$!   Wait queue: 2
$! and another variation:
$!'JCCLML - BBC301' 313033434242202D204C4D4C43434A
$! 'JCC$LCLP01_T' 545F3130504C434C2443434A
$!  'JCC$Loader Sequence Number' 7265626D754E2065636E657571655320726564616F4C2443434A
$!   Wait queue: 1
$	    if f$extract(0,9,Rsn_ASCII_0) .eqs. "JCCLML - " .and. -
			RsnLen_0 .eq. 15 .and. -
		f$extract(0,4,Rsn_ASCII_1) .eqs. "JCC$" .and. -
		f$extract(RsnLen_1-2,2,Rsn_ASCII_1) .eqs. "_T" .and. -
-! Database name may vary in length!		RsnLen_1 .eq. 12 .and. -
		( Rsn_ASCII_2 .eqs. "JCC$Deadman" .or. -
		  Rsn_ASCII_2 .eqs. "JCC$Loader Sequence Number") .and. -
		ConvQueue .eq. 0 .and. WaitQueue .le. 2 then goto loop_lis
$!
$	endif	!End of Depth 2
$!
$ past_filter:
$! End of filter section
$!
$! Here, we've gathered the information for a single lock-queue incident
$! (including all the levels of resource names, in case the lock queue is
$!  on a sub-lock, potentially down several layers of sub-resource names).
$!
$! Output the information.  As we do so, try to decode the meaning of
$! the information in resource names at each level.
$!
$! Output a nodename heading if we haven't already
$	if .not. node_heading
$	then
$		write ls2 ""	!Blank line
$		write ls2 "-----  ",node,":  -----"	!Nodename
$		node_heading = "TRUE"	!Now we've output a heading
$	endif
$!
$!!	if depth .eq. 0 then -
$	write ls2 ""	!Separator between separate resource trees
$!
$! Display the information for each level, starting with the root resource.
$!
$	d = 0
$ depth_loop:
$!
$! Display the resource name in ASCII and hexadecimal
$!
$	write ls2 f$extract(0,d,spaces),-
		"'",Rsn_ASCII_'d',"' ",Rsn_Hex_'d'
$!
$! Decode the meaning of resource names where possible
$!
$	if d .eq. 0
$	then	!Root-level resource name
$!
$! Try to determine what type of lock this is, and whether it is associated with
$! a particular file, disk, or node
$	    if f$extract(0,4,Rsn_ASCII_0) .eqs. "RMS$"
$	    then	!RMS file lock tree
$		devlocknam = f$edit(f$extract(11,12,Rsn_ASCII_0),"TRIM")	!Disk volume name (usually)
$! @@@@ Here, we really need to search through all mounted Files-11 disk volumes
$! looking for the one that has a device lock name of this value, rather than
$! using this value as the volume name.  However, as a short cut, we can simply
$! check that the volume with this volume name has the same device lock name,
$! and if so, we can skip the search and use this as the volume name for the
$! call to FILE_ID_TO_NAME.COM. (At present, we simply use this directly as the
$! volume name, so the volume name shown will be stale if it was changed.) @@@@
$		filn = HexByte_10_0 + HexByte_6_0 + HexByte_5_0	!File number in INDEXF.SYS
$		seq = HexByte_8_0 + HexByte_7_0	!Sequence number
$		rvn = HexByte_9_0
$		write ls2 -	!f$extract(0,d,spaces),-	!(d=0 here)
			"    RMS lock tree for file [",-
				%x'filn',",",%x'seq',",",%x'rvn',-
			"] on volume ",devlocknam
$		call file_id_to_name %x'filn' DISK$'devlocknam'
$		if filespec .nes. "" then -
			write ls2 "    File specification: ",filespec
$	    endif
$	    if f$extract(0,6,Rsn_ASCII_0) .eqs. "F11B$v"
$	    then	!XQP Files-11 volume allocation lock
$		volnam = f$edit(f$extract(6,12,Rsn_ASCII_0),"TRIM")	!Disk volume name
$! The usual caveats apply here about volume name changes and how this is
$! really the Lock Volume Name and might be different from the current name
$		write ls2 -	!f$extract(0,d,spaces),-	!(d=0 here)
			"    Files-11 Volume Allocation lock for volume ",volnam
$	    endif
$	    if f$extract(0,6,Rsn_ASCII_0) .eqs. "F11B$a"
$! F11B$aVolnam123456filr  where 'filr' is the Lock Basis: File#, RVN
$!       123456789012
$! 1234567890123456789012
$	    then	!XQP Files-11 File Access Arbitration lock
$		volnam = f$edit(f$extract(6,12,Rsn_ASCII_0),"TRIM")	!Disk volume name
$! The usual caveats apply here about volume name changes and how this is
$! really the Lock Volume Name and might be different from the current name
$		filn = HexByte_21_0 + HexByte_20_0 + HexByte_19_0
$		rvn = HexByte_22_0
$		write ls2 -	!f$extract(0,d,spaces),-	!(d=0 here)
			"    Files-11 File Access Arbitration lock for file [",-
			%x'filn',",*,",%x'rvn',"] on volume ",volnam
$		call file_id_to_name %x'filn' DISK$'volnam'
$		if filespec .nes. "" then -
			write ls2 "    File specification: ",filespec
$	    endif
$! System ID Lock
$	    if f$extract(0,10,Rsn_ASCII_0) .eqs. "SYS$SYS_ID"
$	    then	!System ID Lock
$		scssystemidx = HexByte_12_0 + HexByte_11_0
$		scssystemid = %x'scssystemidx'
$		node_area = scssystemid / 1024
$		node_number = scssystemid - (1024 * node_area)
$		write ls2 -	!f$extract(0,d,spaces),-	!(d=0 here)
			"    System ID Lock for node with SCSSYSTEMID ",-
			scssystemid," (DECnet address ",node_area,".",node_number,")"
$	    endif
$! Rdb
$!'....D....B07_DB01DBXD.......' 0000000100130044584244313042445F3730420200000044000000DD
$!          123456789.12
$! 0123456789.123456789.1234567
$	    if f$extract((RsnLen_0*2)-8,8,Rsn_Hex_0) .eqs. "000000DD" .and. -
		f$extract(4,1,Rsn_ASCII_0) .eqs. "D" .and. RsnLen_0 .eq. 28
$	    then	!Rdb database lock tree
$		write ls2 -	!f$extract(0,d,spaces),-	!(d=0 here)
			"    Rdb lock tree for database ",-
				f$extract(9,12,Rsn_ASCII_0)
$	    endif
$	else	!Child lock of some level (d<>0)
$! Rdb:
$	    if f$extract((RsnLen_0*2)-8,8,Rsn_Hex_0) .eqs. "000000DD" .and. -
		f$extract(4,1,Rsn_ASCII_0) .eqs. "D" .and. RsnLen_0 .eq. 28
$	    then	!Rdb sub-lock
$!'....D....B07_DB01DBXD.......' 0000000100130044584244313042445F3730420200000044000000DD
$!    Rdb lock tree for database B07_DB01DBXD
$! 'S...Y.' 015900000053
$!     STAREA: Storage Area
$!  'B...y.' 047900000042
$!      LAREA: Logical Area
$!   'L.........y.' 047900000000FFFC0000004C
$!       PLN: DBKEY (record)
$!    Wait queue: 1
$! or
$!'....D....B09_DB01DBXD.......' 00000007002E0044584244313042445F3930420200000044000000DD
$! 'S.....' 009F00000053
$!     STAREA: Storage Area
$!  'p.....' 000100000070
$!      LOGFIL: LOGFIL list
$!   'P...{...' 00018A7B00000050
$!       PNO: Page
$!    Wait queue: 1
$		code = f$extract(0,1,Rsn_ASCII_'d')	!First byte is lock-type code
$		msg = ""	!Indecipherable by default
$		if code .eqs. "A" then msg = "AIJ: After-Image Journal lock"
$		if code .eqs. "B" then msg = "LAREA: Logical Area"
$		if code .eqs. "C" then msg = "SAC: Snapshot Area Cursor"
$!! at root !!	if code .eqs. "D" then msg = "DB: Database"
$		if code .eqs. "E" then msg = "FILID: FILID entry"
$		if code .eqs. "F" then msg = "FRZ: Database Freeze"
$		if code .eqs. "G" then msg = "TSNBLK: TSN Block lock"
$		if code .eqs. "H" then msg = "RTUPB: RTUPB list"
$		if code .eqs. "I" then msg = "MONID: Remote Monitor ID"
$		if code .eqs. "J" then msg = "BCKAIJ: AIJ Backup"
$		if code .eqs. "K" then msg = "DBK_SCOPE: Database Key Scope"
$		if code .eqs. "L" then msg = "PLN: DBKEY (record)"
$		if code .eqs. "M" then msg = "MONITOR: Unique Monitor lock"
$		if code .eqs. "N" then msg = "ACTIVE: ACTIVE bitmap"
$		if code .eqs. "O" then msg = "CLOSE: Database Close"
$		if code .eqs. "P" then msg = "PNO: Page"
$		if code .eqs. "Q" then msg = "QUIET: Quiet"
$		if code .eqs. "R" then msg = "SEQBLK: Database SEQBLK block"
$		if code .eqs. "S" then msg = "STAREA: Storage Area"
$		if code .eqs. "T" then msg = "TRM: Stream Termination"
$		if code .eqs. "U" then msg = "CLIENT: Client specific"
$		if code .eqs. "V" then msg = "MEMBIT: Cluster Membership"
$		if code .eqs. "W" then msg = "ACCESS: Database Access"
$		if code .eqs. "X" then msg = "NOWAIT: nowait transaction signal"
$		if code .eqs. "Y" then msg = "KROOT: KROOT block"
$		if code .eqs. "Z" then msg = "BUGCHK: Bugcheck"
$		if code .eqs. "[" then msg = "RCSWAITRQST: RCS wait for request completion"
$		if code .eqs. "\" then msg = "STATRQST: Process statistics request"
$		if code .eqs. "]" then msg = "BCKAIJ_SPD: AIJ Backup Suspend"
$		if code .eqs. "^" then msg = "AIJHWM: AIJ High Water Mark"
$		if code .eqs. "_" then msg = "REL_GRIC_REQST: Row Cache release GRICs request"
$		if code .eqs. "`" then msg = "REL_GRIC_CMPLT: Row Cache release GRICs completed"
$		if code .eqs. "a" then msg = "AIJOPEN: AIJ journal open channel"
$		if code .eqs. "b" then msg = "CPT: Corrupt Page Table"
$		if code .eqs. "c" then msg = "GBL_CKPT: Global Checkpoint"
$		if code .eqs. "d" then msg = "RO_L1: Snap Truncation L1"
$		if code .eqs. "e" then msg = "RW_L2: Snap Truncation L2"
$		if code .eqs. "f" then msg = "AIJDB: AIJ journal control"
$		if code .eqs. "g" then msg = "AIJFB: AIJ journal info"
$		if code .eqs. "h" then msg = "FIB: Fast Incremental Backup"
$		if code .eqs. "i" then msg = "ROOT_AREA: Dummy Root Area lock"
$		if code .eqs. "j" then msg = "AIJSWITCH: AIJ journal switch-over"
$		if code .eqs. "k" then msg = "RMUCLIENT: RMU utility client"
$		if code .eqs. "l" then msg = "AIJLOGSHIP: AIJ Log Shipping"
$		if code .eqs. "m" then msg = "RCACHE: Record Cache entry"
$		if code .eqs. "n" then msg = "AIJLOGMSG: AIJ Log Message"
$		if code .eqs. "o" then msg = "RUJBLK: RUJBLK entry"
$		if code .eqs. "p" then msg = "LOGFIL: LOGFIL list"
$		if code .eqs. "q" then msg = "ALS: ALS Activation"
$		if code .eqs. "r" then msg = "REL_AREAS: Release Areas"
$		if code .eqs. "s" then msg = "GBPT_SLOT: Global BPT slot"
$		if code .eqs. "t" then msg = "CHAN: File Channel"
$		if code .eqs. "u" then msg = "UTILITY: Utility lock"
$		if code .eqs. "v" then msg = "DASHBOARD: Dashboard Notify"
$		if code .eqs. "w" then msg = "DBR: Database Recovery serialization"
$		if code .eqs. "x" then msg = "CLTSEQ: Client Sequence block"
$		if code .eqs. "y" then msg = "AIPQHD: AIP larea synch"
$		if code .eqs. "z" then msg = "RCSREQUEST: RCS Request / RCS Submittor"
$		if msg .nes. "" then -
$			write ls2 f$extract(0,d+4,spaces),msg
$	    endif
$	endif	!End of decoding for root locks
$!
$	if d .eq. 1		!Child lock of a root lock
$	then
$	    if f$extract(0,4,Rsn_ASCII_0) .eqs. "RMS$"
$	    then	!Child of an RMS File Lock
$		if RsnLen_1 .eq. 4 then -
$			write ls2 f$extract(0,d,spaces),-
		"    RMS bucket lock, on bucket starting at VBN %x",-
				Rsn_Hex_1
$		if RsnLen_1 .eq. 8 then -
$			write ls2 f$extract(0,d,spaces),-
		"    RMS record lock, on RFA %x",-
				Rsn_Hex_1
$	    endif
$	    if f$extract(0,6,Rsn_ASCII_0) .eqs. "F11B$v"
$	    then	!Child of a Files-11 Volume Allocation Lock
$		if f$extract(0,6,Rsn_ASCII_1) .eqs. "F11B$s"
$		then	!XQP Files-11 File Serialization lock:
$!				 F11B$s{lockbasis}
$!				       1234
$!				 1234567890
$		    volnam = f$edit(f$extract(6,12,Rsn_ASCII_0),"TRIM")	!Disk volume name
$! The usual caveats apply here about volume name changes and how this is
$! really the Lock Volume Name and might be different from the current name
$		    filn = HexByte_9_1 + HexByte_8_1 + HexByte_7_1
$		    rvn = HexByte_10_1
$		    write ls2 f$extract(0,d,spaces),-
		"    Files-11 File Serialization lock for file [",-
		%x'filn',",*,",%x'rvn',"] on volume ",volnam
$		    call file_id_to_name %x'filn' DISK$'volnam'
$		    if filespec .nes. "" then -
			write ls2 f$extract(0,d,spaces),-
			    "    File specification: ",filespec
$		endif
$	    endif
$	    if f$extract(0,10,Rsn_ASCII_0) .eqs. "SYS$SYS_ID"
$	    then	!Child of a System ID Lock
$		if f$extract(0,4,Rsn_ASCII_1) .eqs. "RMS$"
$		then	!This is an RMS Global Buffer lock
$		    devlocknam = f$edit(f$extract(11,12,Rsn_ASCII_1),"TRIM")	!Disk volume name (usually)
$! @@@@ Here, because it is possible to change the volume name on a mounted
$! volume, we really need to search through all mounted Files-11 disk volumes
$! looking for the one that has a device lock name of this value, rather than
$! using this value as the volume name.  However, as a short cut, we can simply
$! check that the volume with this volume name has the same device lock name,
$! and if so, we can skip the search and use this as the volume name for the
$! call to FILE_ID_TO_NAME.COM. (At present, we simply use this directly as the
$! volume name, so the volume name shown will be stale if it was changed.) @@@@
$		    filn = HexByte_10_1 + HexByte_6_1 + HexByte_5_1	!File number in INDEXF.SYS
$		    seq = HexByte_8_1 + HexByte_7_1	!Sequence number
$		    rvn = HexByte_9_1
$		    write ls2 f$extract(0,d,spaces),-
			"    RMS Global Buffer Section Lock for file [",-
			%x'filn',",",%x'seq',",",%x'rvn',-
			"] on volume ",devlocknam
$		    call file_id_to_name %x'filn' DISK$'devlocknam'
$		    if filespec .nes. "" then -
			write ls2 f$extract(0,d,spaces),-
			    "    File specification: ",filespec
$	        endif
$	    endif
$	endif	!End of decoding for 1-parent locks
$!
$	if d .lt. depth
$	then	!More levels to display/decode
$		d = d + 1
$		goto depth_loop		!Go check the next depth level
$	endif
$!
$! Display the extent of queue lengths found on this resource
$!
$	if ConvQueue .gt. 0
$	then	!Convert queue found
$	    if WaitQueue .gt. 0
$	    then	!Wait queue also found
$		write ls2 f$extract(0,depth,spaces),-
	f$fao(" Convert queue: !AS, Wait queue: !AS",ConvQueue,WaitQueue) 
$	    else	!Convert queue alone
$		write ls2 f$extract(0,depth,spaces),-
		    f$fao(" Convert queue: !AS",ConvQueue)
$	    endif
$	else	!Wait queue alone
$	    write ls2 f$extract(0,depth,spaces),-
		f$fao(" Wait queue: !AS",WaitQueue) 
$	endif
$!
$	goto loop_lis	!Read the next line of input
$!
$!
$ eof_lis:
$	if .not. node_completed
$	then
$		write ls2 ""
$		write ls2 " ***"
$		write ls2 " *** Node ",node," data collection failed."
$		write ls2 " ***"
$	endif
$	close lis
$	close ls2
$	delete 'temp_file'_lis;*	!!!! Save output
$ type 'output_file'
$!
$!
$!!! show time
$	write sys$output -
"================================== All done. =================================="
$!
$ cleanup:
$! If we turned on OPER priv, turn it off again
$	if OPER_TURNED_ON then set process/privilege=(noOPER)
$	exit
$!
$!
$! Given a file ID and disk device name, find out the file name
$!
$!	P1: File number from File ID (decimal, or hex with %X prefix)
$!	P2: Disk device on which this file is located
$!
$!	Return file specification in global symobl 'filespec'
$!
$ file_id_to_name: subroutine
$	old_verify = f$verify('f$trnlnm("file_id_to_name$verify")'+0)
$	if .not. f$getdvi("''p2'","EXISTS")
$	then
$		write sys$output "    Can't find device ''p2'"
$		goto error_exit
$	endif
$	if f$locate(":",p2) .ne. f$length(p2) then p2 = p2 - ":"
$! Initialization
$	debug = "!"	! "!" to disable debug messages; "" to enable messages
$	pid = f$getjpi("","PID")
$	temp_file := file_id_to_name_temp_'pid'.temp_file
$	maxfiles = f$getdvi("''p2'","MAXFILES")
$	cluster = f$getdvi("''p2'","CLUSTER")
$! Look up the file itself
$	call lookup_file 'p1' 'p2'
$	if filename .eqs. "?.?;?" then goto error_exit
$	initial_filename = filename	!Remember this filespec
$! Look up its directory tree
$	filespec = "]" + filename
$ dir_loop:
$	call lookup_file 'dir_file_id' 'p2'
$	if filename .eqs. "?.?;?" then goto error_exit
$	if filename .eqs. "000000.DIR;1" then goto dir_done
$! If we're just backtracking from extension file headers, keep on looking
$	if filename .eqs. initial_filename .and. -
    f$locate(".DIR;1",filename) .eq. f$length(filename) then goto dir_loop
$	dot = "."
$	if f$extract(0,1,filespec) .eqs. "]" then dot = ""
$	filespec = (filename - ".DIR;1") + dot + filespec
$	goto dir_loop
$ dir_done:
$	if f$extract(0,1,filespec) .eqs. "]" then -
		filespec = "000000" + filespec	!File is in the MFD
$	filespec == p2 + ":[" + filespec
$ !!!	write sys$output "    File specification: ",filespec
$ 	goto done
$ error_exit:
$	filespec == ""
$ done:
$	if old_verify then set verify
$	endsubroutine
$!
$! Look up a file by File ID and return its name and the File ID of the
$! directory it is in
$!
$ lookup_file: subroutine
$	x = ((maxfiles+4095) / 4096) + (4 * cluster) + 'p1'
$	dump/file_header/block=(start:'x',count:1)/output='temp_file' -
		'p2':[000000]indexf.sys
$'debug' type/page 'temp_file';
$	define/user sys$error _nla0:
$	define/user sys$output _nla0:
$	search 'temp_file' "File name:","Back link file identification:"-
		/out='temp_file'_1
$	search_severity = $severity	!Save search result: success=1, fail=3
$	delete 'temp_file';*	!Delete temporary file
$	if search_severity .eq. 3 then goto err	!Didn't find the info we need ->
$	close/nolog temp
$'debug' type/page 'temp_file'_1;
$	open/read temp 'temp_file'_1
$! Read the Directory Backlink record
$	read/end=eof temp rec
$	rec = f$edit(rec,"compress,trim")
$	if f$extract(0,9,rec) .eqs. "Back link"
$	then
$	    dir_backlink = f$element(4," ",rec) - "(" - ")"
$'debug' show symbol dir_backlink	!!!
$	    dir_file_id == f$element(0,",",dir_backlink)	!File number
$'debug' show symbol dir_file_id	!!!
$	else
$	    goto err
$	endif
$! Read the File Name record
$	read/end=eof temp rec
$	rec = f$edit(rec,"compress,trim")
$	if f$extract(0,10,rec) .eqs. "File name:"
$	then
$	    filename == f$element(2," ",rec)
$'debug' show symbol filename	!!!
$	else
$	    goto err
$	endif
$	close temp
$	delete 'temp_file'_1;*
$	goto xit	!Normal exit -->
$ eof:	!Premature end-of-file
$	close temp
$ err:	!Error occurred which prevented looking up the filename and/or directory
$	write sys$output "    Error looking up File Number ",p1," on disk ",p2
$	dir_file_id == "[?]"
$	filename == "?.?;?"
$ xit:
$	endsubroutine
$!-----------------------------------------------------------------------------
$!
$! Build the program
$!
$ BUILD:
$! BUILD_LCKQUE.COM
$!
$! Compile the LCKQUE.MAR program
$!
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$!
$! Done already in mainline code:
$!	proc = f$environment("procedure")
$!	proc_dev = f$parse(proc,,,"DEVICE")
$!	proc_dir = proc_dev + f$parse(proc,,,"DIRECTORY")
$	old_dir = f$environment("DEFAULT")
$	set default 'proc_dir'
$! See if there's a copy of the source out there.  If not, create it.
$	if f$search("LCKQUE.MAR") .eqs. ""
$	then
$		create LCKQUE.MAR
$DECK
        .title  LCKQUE
; The automated build command procedure should pre-pend the appropriate lines
; to handle the symbol definitions shown below.  But if building this alone...
;
; Comment out the following line to compile for 7.1-2 and previous versions,
; or for use on all versions of VMS on VAX.
; Uncomment the following line to compile for 7.2 and following versions on
; Alpha, where the lock manager data structures were moved into 64-bit
; address space (S2) instead of 32-bit address space (S0/S1).
;;S2_SPACE = 1			; KP 5/25/2000
;
; Comment out the following line for VMS version 7.2, 7.2-1, and 7.2-1H1 on
; Alpha.  The LCKMGR developer disallowed executive-mode access along with
; the S2 space changes, but repented for 7.2-2 and 7.3 and following.  If
; executive mode access is allowed, we use that for safety's sake, as the
; worst that should possibly happen due to a bug in this code is an exec-mode
; bugcheck that would result in deletion of the process; if kernel mode access
; is required, we use that instead (and in either case we put into place a
; condition handler, which should trap any accidents and prevent a system
; crash). 
;;KERNEL_ACCESS_REQ = 1			; KP 9/18/2002
;
; Comment out the following line to disable debug code
;;DEBUG = 1
;
;;============================================================================
;;
;; List all the resources whose trees are mastered on this node and which
;; have non-zero-length conversion or wait queues.  Print the resource name
;; along with the lengths of the conversion and wait queues, and also print
;; the resource names of all the parent locks, all the way up the tree to the
;; root, since often the parent resource name(s) are required to uniquely
;; identify the resource (e.g. F11B$v<lockbasis> resource name tells File ID
;; but needs the volume name from the parent resource to identify which disk).
;;
;;						K.Parris 12/99 & 01/2000
;; Modification history:
;;
;;  Update for 7.2-2 and above.  Separate S2 space considerations from
;;  kernel-vs-exec mode access considerations.  Update lock manager version
;;  bitmap info.  Make sure this still works on VAX, too.    KP 09/18/2002
;;
;;  Fix bug which missed lock queues of length 1 by assuming that if head
;;  and tail pointers were the same, the queue was empty.    KP 02/24/00
;;
;;  Write something at successful completion, so we know the process didn't
;;  die with an EXEC mode bugcheck (due to our not locking things down under
;;  the SCS spinlock since we're only reading them).         KP 02/24/00
;;
;;  SCS data structures were moved to S2 space for V7.2.  Added support for
;;  V7.2 and later versions.                                 KP 05/25/00

	.library	/sys$library:lib.mlb/
	.library	/sys$library:starlet.mlb/
	$rsbdef		;Include Resource Block definitions

;;
;; Status-check macro
;;
	.MACRO	CHECK_STATUS, LOC=R0, ?NO_ERROR1$
        BLBS	LOC, NO_ERROR1$	;Branch if no error
	.IF DIFFERENT LOC, R0
        MOVZWL	LOC, R0		;Move error into R0
	.ENDC
        $EXIT_S	R0		;Exit with error
NO_ERROR1$:			;Return to program flow
	.ENDM	CHECK_STATUS

;;
;; Dump a value to the screen
;;
	.MACRO	VALUE_DUMP, VALU=R0, CTL=FAO_DUMP_CTL	;, ?FAO_DUMP_CTL, ?VALUE, ?NAME, ?NAME_DESC
	MOVL	VALU,VALUE	; Grab value to be dumped
	MOVZBL  #MAXFAO,FAOLEN
; Dump resource name with conversion and wait counts
	$FAO_S	-
		CTRSTR=CTL,-
		OUTLEN=FAOLEN,-
		OUTBUF=FAODESC,-
		P1=VALUE
	CHECK_STATUS			;Check status in R0
; Output the message to SYS$OUTPUT
	PUSHAL 	FAODESC
	CALLS	#1, G^LIB$PUT_OUTPUT	;Output the message
	CHECK_STATUS			;Check status in R0
	.ENDM	VALUE_DUMP
;;
;; Dump a 64-bit value to the screen
;;
	.MACRO	VALUE_DUMP64, VALU=R0, CTL=FAO_DUMP_CTL64 ;, ?FAO_DUMP_CTL, ?VALUE, ?NAME, ?NAME_DESC

	evax_stq VALU,VALUE64	; Grab value to be dumped
	MOVZBL  #MAXFAO,FAOLEN
	$FAO_S	-
		CTRSTR=CTL,-
		OUTLEN=FAOLEN,-
		OUTBUF=FAODESC,-
		P1=VALUE64_1,-
		P2=VALUE64_2
	CHECK_STATUS			;Check status in R0
; Output the message to SYS$OUTPUT
	PUSHAL 	FAODESC
	CALLS	#1, G^LIB$PUT_OUTPUT	;Output the message
	CHECK_STATUS			;Check status in R0
	.ENDM	VALUE_DUMP64

;------------------------------------------------------------------------------
;;
;; Read-only data area
;;
	.PSECT	RO_DATA	QUAD,RD,noWRT,noEXE

; Output format strings
;               Depth ConvQ WaitQ RSNLen ResNam(hex)
FAOCTL:	.ASCID	/!3UL !10UL !10UL !2UB !8XL!8XL!8XL!8XL!8XL!8XL!8XL!8XL/
;               Depth                       RSNLen ResNam(hex)
;                     1234567890 1234567890
FAOCTL2: .ASCID	/!3UB (Parent Resource:)    !2UB !8XL!8XL!8XL!8XL!8XL!8XL!8XL!8XL/
; Control strings for VALUE_DUMP macro
  .IF DF DEBUG
FAO_DUMP_CTL:	.ASCID	/Dump: !8XL/
FAO_DUMP_CTL64:	.ASCID	/Dump: !8XL.!8XL/
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
FAO_RRSFL:	.ASCID	/Root RSB List Forward Link: !8XL/
FAO_RootRSB:	.ASCID	/Root RSB: !8XL/
FAO_CKQ_ARG:	.ASCID	/Chk_Queues: Arg RSB: !8XL/
FAO_CKT_ARG:	.ASCID	/Chk_Tree: Arg RSB: !8XL/
FAO_Parent:	.ASCID	/Parent RSB: !8XL/
FAO_RSB:	.ASCID	/RSB: !8XL/
FAO_SRS:	.ASCID	/SRS: !8XL/
FAO_INITIAL:	.ASCID	/Initial link: !8XL/
FAO_NEXT:	.ASCID	/Next link: !8XL/
FAO_CVTQFL:	.ASCID	/CvtQFL: !8XL/
FAO_WTQFL:	.ASCID	/WtQFL: !8XL/
    .IFF	; LckMgr data structures in S2 (64-bit) address space
FAO_RRSFL:	.ASCID	/RSB Queue Forward Link: !8XL.!8XL/
FAO_CKQ_ARG:	.ASCID	/Chk_Queues: Arg RSB: !8XL.!8XL/
FAO_CKT_ARG:	.ASCID	/Chk_Tree: Arg RSB: !8XL.!8XL/
FAO_RootRSB:	.ASCID	/Root RSB: !8XL.!8XL/
FAO_Parent:	.ASCID	/Parent RSB: !8XL.!8XL/
FAO_RSB:	.ASCID	/RSB: !8XL.!8XL/
FAO_SRS:	.ASCID	/SRS: !8XL.!8XL/
FAO_INITIAL:	.ASCID	/Initial link: !8XL.!8XL/
FAO_NEXT:	.ASCID	/Next link: !8XL.!8XL/
FAO_CVTQFL:	.ASCID	/CvtQFL: !8XL.!8XL/
FAO_WTQFL:	.ASCID	/WtQFL: !8XL.!8XL/
    .ENDC ;NDF S2_SPACE
FAO_MstCSID:	.ASCID	/Master CSID: !8XL/
FAO_Depth:	.ASCID	/Depth: !UL/
FAO_RefCnt:	.ASCID	/RefCnt: !UL/	; Reference count (# of sub-resources)
  .ENDC ;DF DEBUG

Done_Msg_Desc:	.ASCID	"LCKQUE done."
;------------------------------------------------------------------------------
;;
;; Read-write data area
;;
	.PSECT	RW_DATA	QUAD,RD,WRT,noEXE

; Data items about each resource with locks waiting
	.ALIGN	LONG
Depth:	.BLKL		; Depth of sub-resource within tree
	.ALIGN	LONG
ConvQ:	.BLKL		; Length of conversion queue
	.ALIGN	LONG
WaitQ:	.BLKL		; Length of wait queue
	.ALIGN	LONG
RSNLen: .BLKL		; Resource name length in bytes
	.ALIGN	LONG
ResNam: .BLKB	32	; Resource name itself

  .IF DF DEBUG
Value:	.BLKL	1	; Register buffer for debug calls
    .IF DF S2_SPACE	; LckMgr data structures in S2 (64-bit) address space
Value64:		; 64-bit Register buffer for debug calls
Value64_2:	.BLKL	1
Value64_1:	.BLKL	1
    .ENDC ;DF S2_SPACE
  .ENDC ;DF DEBUG
;
; Stuff for $FAO calls.
;
MAXFAO = 132			;Maximum $FAO buffer length
	.ALIGN	QUAD
FAODESC:			;$FAO descriptor
FAOLEN:	.BLKL			;$FAO output buffer length
	.ADDRESS	FAOBUF	;address of buffer
FAOBUF:	.BLKB	MAXFAO		;132-character buffer

;------------------------------------------------------------------------------
;;
;;  Executable code area
;;
	.PSECT	CODE	QUAD,RD,noWRT,EXE

;;;;===========================================================================
;;;;
;;;; Main program
;;;;
;;;;===========================================================================
        .ENTRY  START,^M<>


; Lock manager data structures were moved into S2 space for V7.2.  This means
; several field widths had to be changed for 64-bit addressing.
;
; Check to ensure the version of code we're running matches the running system,
; and exit with an error if not:
;        %SYSTEM-F-SYSVERDIF, system version mismatch; please relink
; Although "relink" is a bit misleading, since one may need to actually
; determine the extent of lock manager changes, determine whether or not they
; break any usage this program makes of the data structures, then edit this
; program, recompile, and relink; hopefully the error message
;     "system version mismatch"
; will be enough of a clue to get the job done.
;
; Check for a matching version of lock manager code
	.IIF	NDF,SS$_SYSVERDIF,    $SSDEF
	.IIF	NDF,SYS$K_BASE_IMAGE, $SYSVERSIONDEF
	MOVAL	G^SYS$VERSION_BEGIN,R0
        MOVZBL	#SYS$K_CLUSTERS_LOCKMGR,R1
	MOVL	4(R0)[R1],R2
    .IF NDF 	S2_SPACE
	lckmgr_version712 = ^x00010080	;V7.1-2
	CMPL	#lckmgr_version712,R2
        BEQL    1$
;; We may also need masks for VAX equivalents here...    @@@@
    .IFF        ;NDF S2_SPACE
	lckmgr_version72  = ^x00010100	;V7.2
	lckmgr_version73  = ^x00010110	;V7.3
	lckmgr_version731 = ^x00010120	;V7.3-1
	CMPL	#lckmgr_version72,R2
        BEQL    1$
	CMPL	#lckmgr_version73,R2
        BEQL    1$
	CMPL	#lckmgr_version731,R2
        BEQL    1$
    .ENDC       ;NDF S2_SPACE
        MOVL    #SS$_SYSVERDIF,R0	; Must recompile/relink
	$EXIT_S	R0
1$:	

; Because the RSBs are protected from user-mode access, we must
; gather data from the Root RSB chain while in Exec or Kernel mode.
;
; The data structures are readable from Exec mode on VAX, and on Alpha for
; 7.1-2 and prior versions, or 7.2-2 and above, but only accessible from
; Kernel mode for 7.2, 7.2-1, and 7.2-1H1, so we must use Kernel mode for
; those versions.
    .IF NDF 	KERNEL_ACCESS_REQ	;VAX, or Pre-7.2, or 7.2-2 and above
	$CMEXEC_S ROUTIN=WORK	; Call serious-work routine in Exec mode
    .IFF        ;NDF KERNEL_ACCESS_REQ	;V7.2 flavors that denied Exec access
	$CMKRNL_S ROUTIN=WORK	; Call serious-work routine in Kernel mode
    .ENDC       ;NDF KERNEL_ACCESS_REQ
	CHECK_STATUS			;Check status in R0

; Indicate we completed successfully.
	PUSHAL 	Done_Msg_Desc
	CALLS	#1, G^LIB$PUT_OUTPUT	;Output the message
;;	CHECK_STATUS			;Check status in R0
;; Status gets checked on image exit anyway...
	$EXIT_S	R0
;
	RET	; So Alpha macro compiler knows this routine has ended
;
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
	.ENTRY	WORK,^M<R2,R3,R4,R5,R6,R7,R8,R9,R10>

; Look through the chain of root RSBs, checking each resource and sub-resource.

; (For simplicity here, we dump them in arbitrary order and use DCL code
; later for the work of identifying files by resource name and so forth.)
;
; We don't bother to raise IPL or synchronize with spinlocks because we're only
; reading, and certainly don't want to add to MP_Synch time.
;
; But that implies that we end up off in the weeds sometimes when the data
; structures change underneath us.  While the odds of a root RSB going away
; isn't high, it is still possible, and occasionally does, so we put in place
; an exception handler to catch any ACCVIO that occurs, and prevent the system
; from crashing.  This way, worst case, we may blow away our own process.
; The other thing that sometimes happens due to a lack of synchronization is
; that this program will end up in an infinite loop of abandoned data
; structures.  We catch that case by using a timeout on the SYSMAN command
; which runs this program.
;
; To identify the rare case where we fail to successfully gather data, this
; program outputs a string when it completes successfully.  The DCL shell that
; is run around this looks for that completion string, and if it is not found,
; knows data collection for this node failed, and notifies the user in the
; output.  This only occurs rarely in practice.
;
; Establish an exception handler to catch inadvertent ACCVIOs and prevent a
; system crash in Kernel mode.
;;; Since it won't hurt to also do this in Exec mode, we'll go ahead and do that
;;;    .IF DF KERNEL_ACCESS_REQ		;V7.2 flavors that denied Exec access
	MOVAB	G^EXE$SIGTORET,(FP)	; Establish exception handler
;;;    .ENDC	;DF KERNEL_ACCESS_REQ

    .IF NDF S2_SPACE		;LckMgr data structures in S0/S1 (32-bit) address space
	MOVAL	g^LCK$GL_RRSFL,R7	; Address of Root RSB listhead
  .IF DF DEBUG
    VALUE_DUMP	R7,FAO_RRSFL
  .ENDC ; DF DEBUG
	MOVL	R7,R8			; Make a note of listhead address
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_INITIAL
  .ENDC ; DF DEBUG
    .IFF	;NDF S2_SPACE	;LckMgr data structures in S2 (64-bit) address space
	evax_ldaq R7,g^LCK$GQ_RRSFL	; Address of Root RSB listhead
  .IF DF DEBUG
    VALUE_DUMP64	R7,FAO_RRSFL
  .ENDC ; DF DEBUG
	evax_or	R31,R7,R8		; Make a note of listhead address
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_INITIAL
  .ENDC ; DF DEBUG
    .ENDC	;NDF S2_SPACE

10$:
    .IF NDF S2_SPACE		;LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	(R7),R7			; Next RSB in Root RSB list
  .IF DF DEBUG
    VALUE_DUMP	R7,FAO_NEXT
    moval	<-RSB$L_RRSFL>(r7),r0
    VALUE_DUMP	R7,FAO_RootRSB
  .ENDC ;DF DEBUG
	CMPL	R7,R8			; Back to starting point? (circular queue)
	.branch_unlikely
	BEQL	20$			; Yes --> done
	MOVAB	<-RSB$L_RRSFL>(R7),R9	; Address of base of RSB
  .IF DF DEBUG
    VALUE_DUMP	R9,FAO_RootRSB
  .ENDC ;DF DEBUG
    .IFF	;NDF S2_SPACE	;LckMgr data structures in S2 (64-bit) address space
	evax_or  R31,(R7),R7		; Next RSB in Root RSB list
  .IF DF DEBUG
    VALUE_DUMP64	R7,FAO_NEXT
    evax_ldaq	R0,<-RSB$Q_RRSFL>(R7)
    VALUE_DUMP64	R0,FAO_RootRSB
  .ENDC ;DF DEBUG
	evax_cmpeq	R7,R8,R2	; Back to starting point? (circular queue)
					; Sets R2 to 1 if R7=R8; to 0 if R7<>R8
  .IF DF DEBUG
    VALUE_DUMP64	R2
  .ENDC ;DF DEBUG
	.branch_unlikely
	blbs	R2,20$		; R2 set to 1?  Yes --> done
	evax_ldaq  R9,<-rsb$q_rrsfl>(R7); Address of base of RSB
  .IF DF DEBUG
    VALUE_DUMP64	R9,FAO_RootRSB
  .ENDC ;DF DEBUG
    .ENDC	;NDF S2_SPACE

  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
;    moval	(r9),r0
    movl	r9,r0
    VALUE_DUMP	R0,FAO_RootRSB
    .IFF	; LckMgr data structures in S2 (64-bit) address space
;    evax_ldaq	r0,(r9)
    evax_or	r31,r9,r0
    VALUE_DUMP64	R0,FAO_RootRSB
    .ENDC ;NDF S2_SPACE
    movl	<rsb$l_csid>(r9),r0
    VALUE_DUMP	R0,FAO_MstCSID
  .ENDC ;DF DEBUG

	TSTL	<RSB$L_CSID>(R9)	; Check master node CSID
	BNEQ	10$			; Non-Zero --> Mastered on another node

  .IF DF DEBUG
    moval	(r9),r0
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    VALUE_DUMP	R0,FAO_RootRSB
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    evax_ldaq	r0,(r9)
    VALUE_DUMP64	R0,FAO_RootRSB
    .ENDC ;NDF S2_SPACE
  .ENDC ;DF DEBUG


; Resource tree is mastered on this node.  Check all its sub-resources.
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	PUSHAL	(R9)		; RSB address
	CALLS	#1,CHK_TREE
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R9,R10	; Put argument into R10 for call
	JSB	CHK_TREE
    .ENDC ;NDF S2_SPACE
	BRW	10$		; Get next entry in Root RSB list -->
20$:
	MOVZWL	#SS$_NORMAL,R0	; Return normal status
	RET			; Return to user mode

;
; Check for queues on this Root resource and all its sub-resources
;
; Inputs;
;	R10	RSB address (7.2 or later)
;	4(AP)	RSB address (pre-7.2)
; Outputs:
;	R0	Status
;
; Register usage:
;	R7	Root RSB address
;	R8	Sub-resource list addresses
;	R9	Sub-resource queue head
;

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	.ENTRY	CHK_TREE,^M<R7,R8,R9>
    .IFF	; LckMgr data structures in S2 (64-bit) address space
CHK_TREE:
	.JSB_ENTRY	-
		INPUT=<R10>,-
		OUTPUT=<>,-
		SCRATCH=<>,-
		PRESERVE=<R7,R8,R9,R10>
    .ENDC ;NDF S2_SPACE
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	4(AP),R7	; Grab RSB address from argument list
  .IF DF DEBUG
    VALUE_DUMP	R7,FAO_CKT_ARG
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R10,R7	; Grab RSB address
  .IF DF DEBUG
    VALUE_DUMP64	R7,FAO_CKT_ARG
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE

;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    VALUE_DUMP	R7,FAO_RSB	;;;
    movzbl	<rsb$b_depth>(r7),r0
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    VALUE_DUMP64	R7,FAO_RSB	;;;
    movl	<rsb$l_depth>(r7),r0
    .ENDC ;NDF S2_SPACE
    VALUE_DUMP	R0,FAO_Depth
  .ENDC ;DF DEBUG
;^

;;    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
;;    cmpl	<rsb$l_rtrsb>(r7),r7
;;    beql	3$	; This is its own root RSB -->
;;    .IFF	; LckMgr data structures in S2 (64-bit) address space
;;    evax_cmpeq	<rsb$q_rtrsb>(r7),r7,r24
;;			; Sets R24 to 1 if R7=R8; to 0 if R7<>R8
;;    blbs	r24,3$	; This is its own root RSB -->
;;    .ENDC ;NDF S2_SPACE

;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    movl	<rsb$l_rtrsb>(r7),r0
    VALUE_DUMP	R0,FAO_RootRSB	;;;
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    evax_or	R31,<rsb$q_rtrsb>(r7),r0
    VALUE_DUMP64	R0,FAO_RootRSB	;;;
    .ENDC ;NDF S2_SPACE
  .ENDC ;DF DEBUG
;^

;;3$:

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	movl	<rsb$l_parent>(r7),r0
;;	beql	4$	; No parent -->

  .IF DF DEBUG
    VALUE_DUMP	R0,FAO_Parent	; Parent RSB address ;;;
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,<rsb$q_parent>(r7),r0
;;	evax_beq	r0,4$	; No parent -->
  .IF DF DEBUG
    VALUE_DUMP64	R0,FAO_Parent	; Parent RSB address ;;;
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE

;4$:
; Check for queues on this root resource
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	PUSHL	R7		; Root RSB address
	CALLS	#1,CHK_QUEUES	; Check for queued lock requests
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R7,R10	; Root RSB address
	JSB	CHK_QUEUES	; Check for queued lock requests
    .ENDC ;NDF S2_SPACE
; Now check any sub-resources
;v
  .IF DF DEBUG
    movl	<RSB$L_REFCNT>(r7),r0
    VALUE_DUMP	R0,FAO_RefCnt	; Number of sub-resources
  .ENDC ;DF DEBUG
;^
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVAL	<RSB$L_SRSFL>(R7),R8	; Check for sub-resources
	MOVL	R8,R9			; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP	R9,FAO_INITIAL	; Initial Queue Forward Link
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_ldaq	R8,<RSB$Q_SRSFL>(R7)	; Check for sub-resources
	evax_or	R31,R8,R9			; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP64	R9,FAO_INITIAL	; Initial Queue Forward Link
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE
;;;   movzwl #ss$_normal,r0
;;;   ret	; for testing
10$:
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	(R8),R8		; Follow forward link
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_NEXT	; Next Link
  .ENDC ;DF DEBUG
	CMPL	R8,R9		; Back to starting point? (circular queue)
	BEQL	99$		; Yes --> done
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,(R8),R8	; Follow forward link
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_NEXT	; Next Link
  .ENDC ;DF DEBUG
	evax_cmpeq	R8,R9,R24	; Back to starting point? (circular queue)
					; Sets R24 to 1 if R7=R8; to 0 if R7<>R8
	blbs	R24,99$		; Yes --> done
    .ENDC ;NDF S2_SPACE
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	moval	<-rsb$l_srsfl>(r8),r0
;;;    VALUE_DUMP	R0,FAO_srs ; Sub-resource address
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_ldaq	R0,<-rsb$q_srsfl>(r8)
;;;    VALUE_DUMP64	R0,FAO_srs ; Sub-resource address
    .ENDC ;NDF S2_SPACE

; Here we have a sub-resource.  Check it also.
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	PUSHAL	<-RSB$L_SRSFL>(R8)	; RSB address
	CALLS	#1,CHK_QUEUES
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_ldaq	R10,<-rsb$q_srsfl>(R8)	; RSB address
	JSB	CHK_QUEUES
    .ENDC ;NDF S2_SPACE

;99$: ; for testing
;;   movzwl #ss$_normal,r0 ; for testing
;;   ret	; for testing
	BRB	10$		; Try next resource
99$:
	MOVZWL	#SS$_NORMAL,R0
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	RET
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	RSB
    .ENDC ;NDF S2_SPACE

;
; Check for queues on this resource.
;
; Inputs;
;	R10	RSB address (7.2 or later)
;	4(AP)	RSB address (pre-7.2)
; Outputs:
;	R0	Status
;
; Register usage:
;	R3	Convert queue length
;	R4	Wait queue length
;	R7	RSB address
;	R8	Conversion or wait queue list address
;	R9	Conversion or wait queue head
;
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	.ENTRY	CHK_QUEUES,^M<R2,R3,R4,R5,R6,R7,R8,R9>
    .IFF	; LckMgr data structures in S2 (64-bit) address space
CHK_QUEUES:
	.JSB_ENTRY	-
		INPUT=<R10>,-
		OUTPUT=<>,-
		SCRATCH=<>,-
		PRESERVE=<R3,R4,R7,R8,R9,R10>
    .ENDC ;NDF S2_SPACE

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	4(AP),R7	; Grab RSB address from argument list
  .IF DF DEBUG
    VALUE_DUMP	R7,FAO_CKQ_ARG
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R10,R7	; Grab RSB address from argument register
  .IF DF DEBUG
    VALUE_DUMP64	R7,FAO_CKQ_ARG
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE

;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    VALUE_DUMP	R7,FAO_RSB
    movzbl	<rsb$b_depth>(r7),R0
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    VALUE_DUMP64	R7,FAO_RSB
    movl	<rsb$l_depth>(r7),R0
    .ENDC ;NDF S2_SPACE
    VALUE_DUMP	R0,FAO_Depth
  .ENDC ;DF DEBUG
;^
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	movl	<rsb$l_rtrsb>(r7),r0
;;;v
  .IF DF DEBUG
    VALUE_DUMP	R0,FAO_RootRSB	; Root RSB address
  .ENDC ;DF DEBUG
;;;^
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,<rsb$q_rtrsb>(r7),r0
;;;v
  .IF DF DEBUG
    VALUE_DUMP64	R0,FAO_RootRSB	; Root RSB address
  .ENDC ;DF DEBUG
;;;^
    .ENDC ;NDF S2_SPACE

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    movl	<rsb$l_parent>(r7),r0
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    evax_or	R31,<rsb$q_parent>(r7),r0
    .ENDC ;NDF S2_SPACE
;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    VALUE_DUMP	R0,FAO_Parent	; Parent RSB address
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    VALUE_DUMP64	R0,FAO_Parent	; Parent RSB address
    .ENDC ;NDF S2_SPACE
  .ENDC ;DF DEBUG
;^

; Now check any sub-resources

;v
  .IF DF DEBUG
    movl	<RSB$L_REFCNT>(r7),r0
    VALUE_DUMP	R0,FAO_RefCnt	; Number of sub-resources
  .ENDC ;DF DEBUG
;^

;;   movzwl #ss$_normal,r0
;;   ret	; for testing

; Check for non-zero queue lengths for conversion or wait queues
	CLRL	R3		; Count elements in conversion queue
	CLRL	R4		; Count elements in wait queue
; Check conversion queue
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVAL	<RSB$L_CVTQFL>(R7),R8
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_CVTQFL	; Convert Queue Forward Link
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_ldaq	R8,<RSB$Q_CVTQFL>(R7)
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_CVTQFL	; Convert Queue Forward Link
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE

;;; The code below doesn't appear to save much CPU time, so we don't bother
;;    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
;;	CMPL	R8,(R8)		; Head points to self?
;;	.branch_likely		; Queue is likely to be empty
;;	BEQL	120$		; Head points to itself --> Queue is empty
;;    .IFF	; LckMgr data structures in S2 (64-bit) address space
;;	evax_cmpeq	R8,(R8),R24		; Head points to self?
;;				; Sets R24 to 1 if R7=R8; to 0 if R7<>R8
;;	.branch_likely		; Queue is likely to be empty
;;	blbs	R24,120$ ; Head points to itself --> Queue is empty
;;    .ENDC ;NDF S2_SPACE
;;; The above code doesn't appear to save much CPU time, so we don't bother

; Non-empty conversion queue.  Count the elements (LKBs) in the queue.
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	R8,R9		; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP	R9,FAO_INITIAL	; Initial Queue Forward Link
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R8,R9	; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP64	R9,FAO_INITIAL	; Initial Queue Forward Link
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE
112$:
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	(R8),R8
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_CVTQFL	; Convert Queue Forward Link
  .ENDC ;DF DEBUG
	CMPL	R8,R9		; Back at starting point? (circular queue)
	.branch_likely		; Most queues will be short or empty
	BEQL	118$		; Yes --> Done counting
    .IFF	; LckMgr data structures in S2 (64-bit) address space
;	evax_or	R31,(R8),R8
	evax_ldaq	R8,(R8)
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_CVTQFL	; Convert Queue Forward Link
  .ENDC ;DF DEBUG
	evax_cmpeq	R8,R9,R24 ; Back at starting point? (circular queue)
				  ; Sets R2 to 1 if R7=R8; to 0 if R7<>R8
	.branch_likely		; Most queues will be short or empty
	blbs	R24,118$ ; Yes --> Done counting
    .ENDC ;NDF S2_SPACE
	INCL	R3		; Count this element
	BRB	112$
118$:

; Check wait queue
120$:
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVAL	<RSB$L_WTQFL>(R7),R8
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_WTQFL	; Wait Queue Forward Link
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_ldaq	R8,<RSB$Q_WTQFL>(R7)
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_WTQFL	; Wait Queue Forward Link
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE
;;; The code below doesn't appear to save much CPU time, so we don't bother
;;@@@	CMPL	R8,(R8)		; Head points to self?
;;	.branch_likely		; Queue is likely to be empty
;;@@@	BEQL	130$		; Head points to itself --> Queue is empty
;;; The above code doesn't appear to save much CPU time, so we don't bother
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	R8,R9		; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP	R9,FAO_INITIAL	; Next Link
  .ENDC ;DF DEBUG
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,R8,R9	; Remember starting point
  .IF DF DEBUG
    VALUE_DUMP64	R9,FAO_INITIAL	; Next Link
  .ENDC ;DF DEBUG
    .ENDC ;NDF S2_SPACE
122$:
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	(R8),R8
  .IF DF DEBUG
    VALUE_DUMP	R8,FAO_WTQFL	; Wait Queue Forward Link
  .ENDC ;DF DEBUG
	CMPL	R8,R9		; Back at starting point? (circular queue)
	.branch_likely		; Most queues will be short or empty
	BEQL	128$		; Yes --> Done counting
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,(R8),R8
  .IF DF DEBUG
    VALUE_DUMP64	R8,FAO_WTQFL	; Wait Queue Forward Link
  .ENDC ;DF DEBUG
	evax_cmpeq	R8,R9,R24 ; Back at starting point? (circular queue)
				  ; Sets R24 to 1 if R7=R8; to 0 if R7<>R8
	.branch_likely		; Most queues will be short or empty
	blbs	R24,128$ ; Yes --> Done counting
    .ENDC ;NDF S2_SPACE
	INCL	R4		; Count this element
	BRB	122$
128$:
; Done counting queues (if any)
130$:
	ADDL3	R3,R4,R0	; Any locks in queues?
;;;	.branch_likely		; Most locks will not have queues
;;;	BEQL	200$		; No --> done
;;; Needed a helper for the above branch on VAX:
	.branch_unlikely	; Most locks will not have queues
	BNEQ	131$		; Yes --> queues
	BRW	200$		; No --> done
131$:

;
; R7 still points to RSB
;
; If this is not a root resource, first dump the names of all its parent
; resources in the resource hierarchy.
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	<RSB$L_PARENT>(R7),R8
	BEQL	160$	; Root resource --> No parent locks
	PUSHL	R8	; Push parent resource block address
	CALLS	#1,DISPLAY_RESOURCES
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,<RSB$Q_PARENT>(R7),R8
	evax_beq	R8,160$	; Root resource --> No parent locks
	evax_or	R31,R8,R10	; Pass parent resource block address
	JSB	DISPLAY_RESOURCES
    .ENDC ;NDF S2_SPACE
160$:
;
; Output the info about this resource
;
; Common setup for $FAO
	MOVL	R3,ConvQ
	MOVL	R4,WaitQ
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVZBL	<RSB$B_DEPTH>(R7),Depth	; Resource's depth in tree
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	MOVL	<RSB$L_DEPTH>(R7),Depth	; Resource's depth in tree
    .ENDC ;NDF S2_SPACE
	MOVZBL	<RSB$B_RSNLEN>(R7),R9	; Resource name length
	MOVL	R9,RSNLen		; Resource name length
	MOVC3	R9,<RSB$T_RESNAM>(R7),ResNam ; Resource name text
; Note: R0-R6 are clobbered by MOVC3
	MOVZBL  #MAXFAO,FAOLEN
; Dump resource name with depth plus conversion and wait counts
	$FAO_S	-
		CTRSTR=FAOCTL,-
		OUTLEN=FAOLEN,-
		OUTBUF=FAODESC,-
		P1=Depth,-
		P2=ConvQ,-
		P3=WaitQ,-
		P4=RSNLen,-
		P5=ResNam+28,-
		P6=ResNam+24,-
		P7=ResNam+20,-
		P8=ResNam+16,-
		P9=ResNam+12,-
		P10=ResNam+8,-
		P11=ResNam+4,-
		P12=ResNam
	CHECK_STATUS			;Check status in R0
; Output the message to SYS$OUTPUT
	PUSHAL 	FAODESC
	CALLS	#1, G^LIB$PUT_OUTPUT	;Output the message
	CHECK_STATUS			;Check status in R0

200$:
	MOVZWL	#SS$_NORMAL,R0
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	RET
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	RSB
    .ENDC ;NDF S2_SPACE

;
; Display resource tree of all parent resources up from this RSB
;
; Inputs;
;	R10	RSB address (7.2 or later)
;	4(AP)	RSB address (pre-7.2)
; Outputs:
;	R0	Status
;
; Register usage:
;	R7	RSB address
;	R9	Resource name length
;
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	.ENTRY	DISPLAY_RESOURCES,^M<R2,R3,R4,R5,R6,R7,R8,R9>
	MOVL	4(AP),R7	; Grab RSB address from argument list
    .IFF	; LckMgr data structures in S2 (64-bit) address space
DISPLAY_RESOURCES:
	.JSB_ENTRY	-
		INPUT=<R10>,-
		OUTPUT=<>,-
		SCRATCH=<>,-
		PRESERVE=<R2,R3,R4,R5,R6,R7,R9,R10>
	evax_or	R31,R10,R7	; Grab RSB address from argument register
    .ENDC ;NDF S2_SPACE

;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    VALUE_DUMP	R7,FAO_RSB
    movzbl	<rsb$b_depth>(r7),r0
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    VALUE_DUMP64	R7,FAO_RSB
    movl	<rsb$l_depth>(r7),r0
    .ENDC ;NDF S2_SPACE
    VALUE_DUMP	R0,FAO_Depth
  .ENDC ;DF DEBUG
;^

; If this resource itself is not a root resource, dump the names of all its
; parent resources first (so we end up with the root resource name dumped 1st).
;v
  .IF DF DEBUG
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
    movl	<rsb$l_parent>(r7),r0
    VALUE_DUMP	R0,FAO_Parent	; Parent RSB address
    .IFF	; LckMgr data structures in S2 (64-bit) address space
    evax_or	R31,<rsb$q_parent>(r7),r0
    VALUE_DUMP64	R0,FAO_Parent	; Parent RSB address
    .ENDC ;NDF S2_SPACE
  .ENDC ;DF DEBUG
;^

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVL	<RSB$L_PARENT>(R7),R8
	BEQL	300$	; No parent lock --> Root resource
	PUSHL	R8	; Push parent resource block address
	CALLS	#1,DISPLAY_RESOURCES	; Call ourselves recursivly
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	evax_or	R31,<RSB$Q_PARENT>(R7),R8
	evax_beq	R8,300$	; No parent lock --> Root resource
	evax_or	R31,R8,R10	; Push parent resource block address
	JSB	DISPLAY_RESOURCES	; Call ourselves recursivly
    .ENDC ;NDF S2_SPACE
300$:

    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	MOVZBL	<RSB$B_DEPTH>(R7),Depth	; Resource's depth in tree
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	MOVL	<RSB$L_DEPTH>(R7),Depth	; Resource's depth in tree
    .ENDC ;NDF S2_SPACE
	MOVZBL	<RSB$B_RSNLEN>(R7),R9	; Resource name length
	MOVL	R9,RSNLen		; Resource name length
	MOVC3	R9,<RSB$T_RESNAM>(R7),ResNam ; Resource name text
; Note: R0-R6 are clobbered by MOVC3
	MOVZBL  #MAXFAO,FAOLEN
; Dump resource name with depth plus conversion and wait counts
	$FAO_S	-
		CTRSTR=FAOCTL2,-
		OUTLEN=FAOLEN,-
		OUTBUF=FAODESC,-
		P1=Depth,-
		P2=RSNLen,-
		P3=ResNam+28,-
		P4=ResNam+24,-
		P5=ResNam+20,-
		P6=ResNam+16,-
		P7=ResNam+12,-
		P8=ResNam+8,-
		P9=ResNam+4,-
		P10=ResNam
	CHECK_STATUS			;Check status in R0
; Output the message to SYS$OUTPUT
	PUSHAL 	FAODESC
	CALLS	#1, G^LIB$PUT_OUTPUT	;Output the message
	CHECK_STATUS			;Check status in R0

	MOVZWL	#SS$_NORMAL,R0
    .IF NDF S2_SPACE	; LckMgr data structures in S0/S1 (32-bit) address space
	RET
    .IFF	; LckMgr data structures in S2 (64-bit) address space
	RSB
    .ENDC ;NDF S2_SPACE

        .end start
$EOD
$	endif
$!
$	arch = f$getsyi("ARCH_NAME")	!Architecture
$	full_version = f$getsyi("VERSION")	!Version of VMS (i.e. V7.3-1)
$	version = full_version - "."	!Version, minus dot (i.e. V73-1)
$!!!!	num_version = version - "V" - "T" - "X" - "G"
$	num_version = f$extract(1,99,version)	!Strip leading letter (i.e. 73-1)
$	write sys$output "Architecture: ''arch'; VMS version: ''full_version'"
$	imagename := lckque.exe_'arch'_'version'
$	if arch .eqs. "Alpha"
$	then	!Alpha
$		compile_cmd = "macro/migrate"
$		link_cmd = "LINK/SYSEXE/NODEBUG/MAP/EXE=''imagename' lckque"
$	else	!VAX
$		compile_cmd = "macro"
$		link_cmd = "LINK/NODEBUG/MAP/EXE=''imagename' lckque,SYS$SYSTEM:SYS.STB"
$	endif
$!
$	pid = f$getjpi("","PID")
$	macro_symbol_defs_file := build_lckque_temp_'pid'.options_temp
$!
$! Handle versions in which lock manager data structures are in 64-bit (S2)
$! address space
$	if arch .eqs. "Alpha" .and. num_version .ges. "72"
$	then
$		create 'macro_symbol_defs_file'
S2_SPACE = 1	; Lock manager data structures in S2 space
$		write sys$output "Building to support LckMgr data in S2 space"
$	else	! V7.1-2 or earlier version, or all VAX versions
$		create 'macro_symbol_defs_file'
;;;S2_SPACE = 1	; Uncomment if lock manager data structures are in S2 space
$		write sys$output "Building to support LckMgr data in S0/S1 space"
$	endif
$!
$! Handle versions in which lock manager data structures cannot be accessed
$! from Exec mode, and must be accessed from Kernel mode.  This was
$! versions 7.2, 7.2-1, and 7.2-1H1 on Alpha.  Exec-mode access was
$! restored for 7.2-2, 7.3, and following versions.
$!
$	if arch .eqs. "Alpha" .and. -
		(num_version .ges. "72" .and. num_version .lts. "72-2")
$	then
$		append sys$input 'macro_symbol_defs_file'
KERNEL_MODE_REQ = 1	; Kernel-mode access required for lock manager data structures
$		write sys$output "Building to support LckMgr access from Kernel mode"
$	else	! V7.1-2 or earlier version
$		append sys$input 'macro_symbol_defs_file'
;;;KERNEL_MODE_REQ = 1	; Uncomment if lock manager data structures are inaccessible from Exec mode
$		write sys$output "Building to support LckMgr access from Exec mode"
$	endif
$!
$! Compile
$	'compile_cmd'/list=lckque.lis/object=lckque.obj/debug -
		'macro_symbol_defs_file'+lckque.mar
$	delete := delete
$	delete 'macro_symbol_defs_file';*
$! Link
$	'link_cmd'
$!
$	set default 'old_dir'
$	exit
$!-----------------------------------------------------------------------------
$!
$! Run the LCKQUE.MAR program (called via SYSMAN on each node in turn)
$!
$ LCKQUE_RUN:
$! LCKQUE_RUN.COM
$!
$! Gather lock queue data on a give node
$!
$! Author: Keith Parris  parris@encompasserve.org or keithparris@yahoo.com
$! http://www.geocities.com/keithparris/ or http://encompasserve.org/~parris/
$!                                     Version as of VMS 7.3-1  Sept. 2002
$! Done already in mainline code:
$!	proc = f$environment("procedure")
$!	proc_dev = f$parse(proc,,,"DEVICE")
$!	proc_dir = proc_dev + f$parse(proc,,,"DIRECTORY")
$!
$! Although we have an inner-mode handler in place to try to avoid any problems
$! if there were a bug, it's conceivable that since we run in Exec or Kernel
$! mode that we might generate a bugcheck in Exec mode or something.  Normally
$! this would kill the process, worst-case, but...
$! To avoid any possibility of crashing a system in such a case, don't run
$! the program if BUGCHECKFATAL is set to 1, or if SYSTEM_CHECK is on, which
$! implies BUGCHECKFATAL=1 behavior.
$	if f$getsyi("BUGCHECKFATAL") .ne. 0
$	then
$		write sys$output -
	"BUGCHECKFATAL is set to 1.  Data-gathering avoided."
$		exit
$	endif
$	if f$getsyi("SYSTEM_CHECK") .ne. 0
$	then
$		write sys$output -
"SYSTEM_CHECK is non-zero, implying BUGCHECKFATAL=1.  Data-gathering avoided."
$		exit
$	endif
$! First, look for an architecture and VMS-version-specific image
$	arch = f$getsyi("ARCH_NAME")
$	version = f$getsyi("VERSION") - "."	!VMS version
$	imagename := 'proc_dir'LCKQUE.EXE_'arch'_'version'
$	image = f$search(imagename)
$! Failing that, look for a generic image
$	if image .eqs. "" then image = f$search("''proc_dir'LCKQUE.EXE")
$	if image .nes. ""
$	then
$		run 'image'
$	else
$		write sys$output -
	"Image LCKQUE.EXE* not found for OpenVMS ''arch' ''version'"
$! Maybe we should just do a BUILD automatically here...
$	endif
$	exit
